package com.zhili.dashboard.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zhili.dashboard.entity.Device;
import com.zhili.dashboard.mapper.DeviceMapper;
import com.zhili.dashboard.req.TableInfoParam;
import com.zhili.dashboard.service.IDeviceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhili.dashboard.utils.OkHttpCli;
import com.zhili.dashboard.vo.BatteryDataVo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Future;

/**
 * <p>
 * 设备信息 服务实现类
 * </p>
 *
 * @author zyg
 * @since 2022-07-05
 */
@Service
public class DeviceServiceImpl extends ServiceImpl<DeviceMapper, Device> implements IDeviceService {

    @Resource
    OkHttpCli okHttpCli;
    @Resource
    DeviceMapper deviceMapper;

    /**
     * 数据获取地址
     */
    private static final String BATTERY_URL = "/admin/v1/hzBattery";
    /**
     * 故障信息地址
     */
    private static final String SIGNATURE = "AD5C4EC001352FFAC3A03CBA787A7417";

    @Value("${openapi.token-secret}")
    private String tokenSecret;
    @Value("${openapi.url}")
    private String openApiUrl;

    /**
     * 故障信息
     */
    @Async
    public Future<String> getBatteryMsg(String url, String paramJson) {
        Map<String, String> mapHeaders = new HashMap<>();
        mapHeaders.put("token", tokenSecret);
        String response = okHttpCli.doPostTokenJson(url, paramJson, mapHeaders);
        Future<String> result = new AsyncResult<>(response);
        return result;
    }


    /**
     * 同步电池信息
     * 获取上周合众数据，注意间隔时间为7天
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void syncBattery() {
        QueryWrapper<Device> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_delete", 0);
        queryWrapper.select("max(sid) as id");
        Device device = this.getOne(queryWrapper);

        long maxId = 0L;
        if (device != null && device.getId() != null) {
            maxId = device.getId();
        }

        //获取电池信息
        List<String> response = new ArrayList<>();
        try {
            List<Future<String>> futures = new ArrayList<>();
            TableInfoParam param = new TableInfoParam();
            param.setSignature(SIGNATURE);
            param.setInterfaceName("设备基本信息");
            //param.setCondition("update_time <= DATE_ADD(now(), INTERVAL -7 DAY) and id>" + maxId);
            param.setCondition("id>" + maxId);
            param.setTableName("t_battery");
            param.setFields("id,sn,vin,latitude,longitude,district,town,city,province,online_time,update_time,battery_status,battery_core_type");

            futures.add(getBatteryMsg(openApiUrl + BATTERY_URL, JSON.toJSONString(param)));
            //死循环，每隔2000ms执行一次，判断一下这三个异步调用的方法是否全都执行完了。
            while (true) {
                //使用Future的isDone()方法返回该方法是否执行完成
                if (futures.get(0).isDone()) {
                    //如果异步方法全部执行完，跳出循环
                    break;
                }
                //每隔200毫秒判断一次
                Thread.sleep(200);
            }
            for (Future<String> future : futures) {
                String string = future.get();
                response.add(string);
            }
        } catch (Exception e) {
            return;
        }
        // 故障信息解析
        saveBattery(response);
    }

    @Override
    public void updateDevice() {
        int random = (int) (Math.random() + 1);
        Device randomDevice = deviceMapper.getRandomDevice(random);
        baseMapper.insert(randomDevice);
    }


    @Transactional(rollbackFor = Exception.class)
    public void saveBattery(List<String> response) {
        if (response.size() > 0) {
            JSONObject baseData = JSONObject.parseObject(response.get(0));
            List<BatteryDataVo> batteryDataVos = new ArrayList<>();
            if (baseData != null && baseData.get("data") != null) {
                batteryDataVos = JSONObject.parseArray(baseData.get("data").toString(), BatteryDataVo.class);
                //确实存在报警信息
                if (batteryDataVos.size() > 0) {
                    List<Device> deviceList = new ArrayList<>();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    //sn号前
                    batteryDataVos.forEach(o -> {
                        Device device = new Device();
                        device.setSn(o.getSn());
                        device.setProvince(o.getProvince());
                        device.setCity(o.getCity());
                        device.setDistrict(o.getDistrict());
                        device.setTown(o.getTown());
                        device.setLongitude(o.getLongitude());
                        device.setLatitude(o.getLatitude());
                        device.setSid(o.getId());
                        device.setBatteryCoreType(o.getBatteryCoreType());
                        if (o.getBatteryStatus()==null){
                            device.setStatus(4);
                        }else {
                            switch (o.getBatteryStatus()) {
                                case 0:
                                    device.setStatus(2);
                                    break;
                                case 1:
                                case 4:
                                    device.setStatus(3);
                                    break;
                                case 2:
                                case 3:
                                    device.setStatus(1);
                                    break;
                                default:
                                    device.setStatus(4);
                            }
                        }

                        //0-乘用车 1-重卡 2-低速车 3-储能 4-备用电源
                        device.setScene(0);
                        if (device.getSn().startsWith("004PEB")) {
                            device.setScene(1);
                        } else if (device.getSn().startsWith("OCZPE0")) {
                            device.setScene(4);
                        } else if (device.getSn().startsWith("09SPB0")) {
                            device.setScene(3);
                        } else if (device.getSn().startsWith("09UPB6") || device.getSn().startsWith("03HPB0")) {
                            device.setScene(2);
                        }

                        if (device.getSn().startsWith("001PB0") || device.getSn().startsWith("0CZPE0")) {
                            device.setOrganId("113");
                        } else if (device.getSn().startsWith("03HPB0") || device.getSn().startsWith("09SPB0")) {
                            device.setOrganId("114");
                        } else if (device.getSn().startsWith("09SPE0") || device.getSn().startsWith("09UPB6")) {
                            device.setOrganId("112");
                        } else if (device.getSn().startsWith("004PEB") || device.getSn().startsWith("02YPEA")
                                || device.getSn().startsWith("051PE0") || device.getSn().startsWith("OCZPE0")) {
                            device.setOrganId("117");
                        }

                        device.setOnlineFlag(0);
                        //LocalDateTime onlineTime = DateUtil.Timestamp2LocalDateTime(o.getOnlineTime().getTime() + 7 * 24 * 60 * 60 * 1000L);
                        if (o.getOnlineTime() != null) {
                            long l = o.getOnlineTime().getTime() + 7 * 86400000 + 3600000L;
                            if (l < System.currentTimeMillis()) {
                                device.setOnlineFlag(1);
                            }
                        }
                        deviceList.add(device);
                    });
                    if (deviceList.size() > 0) {
                        this.saveBatch(deviceList);
                    }
                }
            }
        }
    }


}
